/* ***************************************************************************
   *  Kindermann                                 WPK Produktentwicklung      *
   *                                             Schuljahr: 2014/15          *
   *                                                                         *
   *                                                                         *
   *                  Ampelschaltung fr den ATtiny 25/45/85                 *
   *                                                                         *
   *                                                                         *
   *                                                                         *
   ***************************************************************************/


// ATMEL ATTINY25/45/85:
//
//                  +-\/-+
//         RESET   1|    |8   Vcc
//           PB3   2|    |7   PB2   LED rot  (A)
//           PB4   3|    |6   PB1   LED gelb (A)
//      	 GND   4|    |5   PB0   LED grn (A)
//                  +----+
//
// ATtiny85 fuse bits:
// Example for 8 MHz internal oscillator divided by 8 which gives 1MHz
// Fuse high byte:
// 0xde = 1 1 0 1 1 1 1 0 --- BODLEVEL0 (Brown out detection)
//        ^ ^ ^ ^ ^ ^ ^------ BODLEVEL1 (Brown out detection)
//        | | | | | +-------- BODLEVEL2 (Brown out detection)
//        | | | | +---------- EESAVE (set to 0 to preserve EEPROM over chip erase)
//        | | | +------------ WDTON (if set to 0, watchdog is always on)
//        | | +-------------- SPIEN (if set to 1, serial programming is disabled)
//        | +---------------- DWEN (if set to 0, debug wire enabled)
//        +------------------ RSTDISBL (if set to 0, RESET pin is disabled)
//
// Brown out detection
// 111 = disabled
// 110 = VCC 1,8V
// 101 = VCC 2,7V
// 100 = VCC 4,3V
//
// Fuse low byte:
// 0x62 = 0 1 1 0 0 0 1 0
//        ^ ^ ^ ^ \--+--/
//        | | | +------- CKSEL 3..0 (8M internal RC)
//        | | +--------- SUT 1..0 (slowly rising power)
//        | +----------- CKOUT (if set to 0, Clock output on PORTB4)
//        +------------- CKDIV8 (if set to 0, divide clock by 8 internally)
//
//CKSEL table
// 0000 = External clock (not crystal oscillator)
// 0001 = 16MHz PLL clock
// 0010 = 8MHz Internal clock

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

#define F_CPU 1000000UL
#include <util/delay.h>

#define BODS 7   //BOD Sleep bit in MCUCR
#define BODSE 2  //BOD Sleep enable bit in MCUCR

uint8_t lcount=0, mcucr1, mcucr2;

void goToSleep(void) {
    // GIMSK |= _BV(INT0);                    //enable INT0
    // MCUCR &= ~(_BV(ISC01) | _BV(ISC00));   //INT0 on low level
    ACSR |= _BV(ACD);                         //disable the analog comparator
    ADCSRA &= ~_BV(ADEN);                     //disable ADC
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);
    sleep_enable();
    //turn off the brown-out detector.
    //must have an ATtiny45 or ATtiny85 rev C or later for software to be able to disable the BOD.
    //current while sleeping will be <0.5uA if BOD is disabled, <25uA if not.
    cli();
    mcucr1 = MCUCR | _BV(BODS) | _BV(BODSE);  //turn off the brown-out detector
    mcucr2 = mcucr1 & ~_BV(BODSE);
    MCUCR = mcucr1;
    MCUCR = mcucr2;
    sei();                         //ensure interrupts enabled so we can wake up again
    sleep_cpu();                   //go to sleep
    cli();                         //wake up here, disable interrupts
    // GIMSK = 0x00;               //disable INT0
    sleep_disable();
    sei();                         //enable interrupts again (but INT0 is disabled from above)
}


// Hauptprogramm
int main( void )
  {

  DDRB = _BV(PB0) | _BV(PB1) | _BV(PB2);

  for(;;)
    {
    int unsigned j;
    lcount += 1;
    PORTB = _BV(PB2);
    PORTB &= ~_BV(PB1);

    for (j=0; j<(8*3); j++)
      {
      _delay_ms(200);
      }

    PORTB = _BV(PB2) | _BV(PB1);

    for (j=0; j<(8*1); j++)
      {
      _delay_ms(200);
      }

     PORTB &= ~_BV(PB2);
     PORTB &= ~_BV(PB1);
     PORTB = _BV(PB0);

    for (j=0; j<(8*3); j++)
      {
      _delay_ms(200);
      }

    PORTB &= ~_BV(PB0);
    PORTB = _BV(PB1);

    for (j=0; j<(8*1); j++)
     {
      _delay_ms(200);
     }

    if (lcount > 5)
     {
     PORTB &= ~_BV(PB1);
     lcount=0; goToSleep(); }
     }

  }



